@@ -2,17 +2,19 @@ from functools import partial
 from typing import Callable, Optional, List
 
 from mypy import message_registry
-from mypy.nodes import StrExpr, IntExpr, DictExpr, UnaryExpr
+from mypy.nodes import Expression, StrExpr, IntExpr, DictExpr, UnaryExpr
 from mypy.plugin import (
-    Plugin, FunctionContext, MethodContext, MethodSigContext, AttributeContext, ClassDefContext
+    Plugin, FunctionContext, MethodContext, MethodSigContext, AttributeContext, ClassDefContext,
+    CheckerPluginInterface,
 )
 from mypy.plugins.common import try_getting_str_literals
 from mypy.types import (
     Type, Instance, AnyType, TypeOfAny, CallableType, NoneType, TypedDictType,
-    TypeVarType, TPDICT_FB_NAMES, get_proper_type
+    TypeVarType, TPDICT_FB_NAMES, get_proper_type, LiteralType
 )
 from mypy.subtypes import is_subtype
 from mypy.typeops import make_simplified_union
+from mypy.checkexpr import is_literal_type_like
 
 
 class DefaultPlugin(Plugin):
@@ -56,6 +58,8 @@ class DefaultPlugin(Plugin):
             return typed_dict_get_callback
         elif fullname == 'builtins.int.__pow__':
             return int_pow_callback
+        elif fullname == 'builtins.int.__neg__':
+            return int_neg_callback
         elif fullname in set(n + '.setdefault' for n in TPDICT_FB_NAMES):
             return typed_dict_setdefault_callback
         elif fullname in set(n + '.pop' for n in TPDICT_FB_NAMES):
@@ -66,6 +70,8 @@ class DefaultPlugin(Plugin):
             return ctypes.array_getitem_callback
         elif fullname == 'ctypes.Array.__iter__':
             return ctypes.array_iter_callback
+        elif fullname == 'pathlib.Path.open':
+            return path_open_callback
         return None
 
     def get_attribute_hook(self, fullname: str
@@ -101,23 +107,55 @@ class DefaultPlugin(Plugin):
 
 
 def open_callback(ctx: FunctionContext) -> Type:
-    """Infer a better return type for 'open'.
+    """Infer a better return type for 'open'."""
+    return _analyze_open_signature(
+        arg_types=ctx.arg_types,
+        args=ctx.args,
+        mode_arg_index=1,
+        default_return_type=ctx.default_return_type,
+        api=ctx.api,
+    )
 
-    Infer TextIO or BinaryIO as the return value if the mode argument is not
-    given or is a literal.
+
+def path_open_callback(ctx: MethodContext) -> Type:
+    """Infer a better return type for 'pathlib.Path.open'."""
+    return _analyze_open_signature(
+        arg_types=ctx.arg_types,
+        args=ctx.args,
+        mode_arg_index=0,
+        default_return_type=ctx.default_return_type,
+        api=ctx.api,
+    )
+
+
+def _analyze_open_signature(arg_types: List[List[Type]],
+                            args: List[List[Expression]],
+                            mode_arg_index: int,
+                            default_return_type: Type,
+                            api: CheckerPluginInterface,
+                            ) -> Type:
+    """A helper for analyzing any function that has approximately
+    the same signature as the builtin 'open(...)' function.
+
+    Currently, the only thing the caller can customize is the index
+    of the 'mode' argument. If the mode argument is omitted or is a
+    string literal, we refine the return type to either 'TextIO' or
+    'BinaryIO' as appropriate.
     """
     mode = None
-    if not ctx.arg_types or len(ctx.arg_types[1]) != 1:
+    if not arg_types or len(arg_types[mode_arg_index]) != 1:
         mode = 'r'
-    elif isinstance(ctx.args[1][0], StrExpr):
-        mode = ctx.args[1][0].value
+    else:
+        mode_expr = args[mode_arg_index][0]
+        if isinstance(mode_expr, StrExpr):
+            mode = mode_expr.value
     if mode is not None:
-        assert isinstance(ctx.default_return_type, Instance)  # type: ignore
+        assert isinstance(default_return_type, Instance)  # type: ignore
         if 'b' in mode:
-            return ctx.api.named_generic_type('typing.BinaryIO', [])
+            return api.named_generic_type('typing.BinaryIO', [])
         else:
-            return ctx.api.named_generic_type('typing.TextIO', [])
-    return ctx.default_return_type
+            return api.named_generic_type('typing.TextIO', [])
+    return default_return_type
 
 
 def contextmanager_callback(ctx: FunctionContext) -> Type:
@@ -365,8 +403,10 @@ def typed_dict_update_signature_callback(ctx: MethodSigContext) -> CallableType:
 
 def int_pow_callback(ctx: MethodContext) -> Type:
     """Infer a more precise return type for int.__pow__."""
-    if (len(ctx.arg_types) == 1
-            and len(ctx.arg_types[0]) == 1):
+    # int.__pow__ has an optional modulo argument,
+    # so we expect 2 argument positions
+    if (len(ctx.arg_types) == 2
+            and len(ctx.arg_types[0]) == 1 and len(ctx.arg_types[1]) == 0):
         arg = ctx.args[0][0]
         if isinstance(arg, IntExpr):
             exponent = arg.value
@@ -380,3 +420,30 @@ def int_pow_callback(ctx: MethodContext) -> Type:
         else:
             return ctx.api.named_generic_type('builtins.float', [])
     return ctx.default_return_type
+
+
+def int_neg_callback(ctx: MethodContext) -> Type:
+    """Infer a more precise return type for int.__neg__.
+
+    This is mainly used to infer the return type as LiteralType
+    if the original underlying object is a LiteralType object
+    """
+    if isinstance(ctx.type, Instance) and ctx.type.last_known_value is not None:
+        value = ctx.type.last_known_value.value
+        fallback = ctx.type.last_known_value.fallback
+        if isinstance(value, int):
+            if is_literal_type_like(ctx.api.type_context[-1]):
+                return LiteralType(value=-value, fallback=fallback)
+            else:
+                return ctx.type.copy_modified(last_known_value=LiteralType(
+                    value=-value,
+                    fallback=ctx.type,
+                    line=ctx.type.line,
+                    column=ctx.type.column,
+                ))
+    elif isinstance(ctx.type, LiteralType):
+        value = ctx.type.value
+        fallback = ctx.type.fallback
+        if isinstance(value, int):
+            return LiteralType(value=-value, fallback=fallback)
+    return ctx.default_return_type
